#pragma once

#include "Pokitto.h"
#include "Particle.h"
#include "Layer.h"
#include "vector"
#include "algorithm"

class Engine
{

private:
    std::vector<Particle> particles;

public:
    Engine()
    {

    }

    void AddParticle(Particle p)
    {
        particles.push_back(p);
    }

    void AddDust(Fix16 x,Fix16 y,int angle)
    {
        Particle p=Particle(x,
                            y,
                            0.0,
                            0.0,
                            60,
                            Particle::Dust);
        particles.push_back(p);
    }

    void AddSmoke(Fix16 x,Fix16 y,int angle)
    {
        Particle p=Particle(x,
                            y,
                            0.0,
                            0.0,
                            60,
                            Particle::Smoke);
        particles.push_back(p);
    }

     void AddRocks(Fix16 x,Fix16 y)
    {
        for(uint16_t i=0; i<10; i++)
        {
            Particle p=Particle(x,
                                y,
                                fix16_from_int(random(1,4)),
                                toRad*fix16_from_int(random(0,360)),
                                10,
                                Particle::Rock);
            AddParticle(p);
        }
    }

    void AddExplosion(Fix16 x,Fix16 y,int16_t angleMin, int16_t angleMax)
    {
        for(uint16_t i=0; i<10; i++)
        {
            Particle p=Particle(x,
                                y,
                                fix16_from_int(random(1,4)),
                                toRad*fix16_from_int(random(angleMin,angleMax)),
                                10,
                                Particle::Sparkle);
            AddParticle(p);
        }
        Particle p=Particle(x,
                            y,
                            0.0,
                            0.0,
                            30,
                            Particle::Explosion);
        AddParticle(p);
    }

    void update(Layer* layer)
    {
        int16_t xcell;
        int16_t ycell;
        Layer::TileExp tOn;
        for(auto& p:particles)
        {
            p.update();

            xcell = p.getX()/CELL_WIDTH;
            ycell = p.getY()/CELL_HEIGHT;
            tOn=layer->getTileExp(xcell,ycell,0,0);

             if(p.getType()==Particle::Dust && tOn.Solid)
            {
                //Destroy particle
                p.setLife(0);
            }

            if(p.getType()==Particle::Bullet && tOn.Solid)
            {
                //Destroy particle
                p.setLife(0);

                //Demage destructible tile life
                layer->damageTile(xcell,ycell,50);

                //Explosion
                Fix16 bulletAngle=fix16_atan2(p.getDXfix16(),p.getDYfix16());
                bulletAngle*=toDegrees;
                int16_t bAngle=fix16_to_int(bulletAngle)-180;
                AddExplosion(p.getXfix16(),p.getYfix16(),bAngle-30,bAngle+30);
            }
        }

        //Remove death particles
        particles.erase(
            std::remove_if(particles.begin(),
                           particles.end(),
                           [](Particle &p)
        {
            return p.isDead();
        }),
        particles.end());
    }

    void draw(Layer* layer)
    {
        //Draw particles
        for(auto& p:particles)
        {
            p.draw(layer->getCamera());
        }
    }


};


