#ifndef PARTICLE_H_INCLUDED
#define PARTICLE_H_INCLUDED

#include "Pokitto.h"



class Particle
{
public:
    typedef enum: uint8_t
    {
        Explosion,
        Bullet,
        Dust,
        Pixels,
        Sparkle,
        Smoke,
        Rock
    } Type;

private:
    Fix16 x;
    Fix16 y;
    Fix16 dx;
    Fix16 dy;
    int8_t life;
    Type type;

public:

    Particle(Fix16 x,Fix16 y,Fix16 speed,Fix16 angle,int8_t life,Type type)
    {
        this->x=x;
        this->y=y;
        this->dx=fix16_mul(fix16_sin(angle),speed);
        this->dy=fix16_mul(fix16_cos(angle),speed);
        this->life=life;
        this->type=type;
    }

    int getX()
    {
        return fix16_to_int(x);
    }

    int getY()
    {
        return fix16_to_int(y);
    }

    Fix16 getXfix16()
    {
        return x;
    }

    Fix16 getYfix16()
    {
        return y;
    }

    Fix16 getDXfix16()
    {
        return dx;
    }

    Fix16 getDYfix16()
    {
        return dy;
    }


    bool isDead()
    {
        return life<1;
    }

    Type getType()
    {
        return type;
    }

    void setLife(int8_t life)
    {
        this->life=life;
    }

    void update()
    {
        x+=dx;
        y+=dy;

        switch(type)
        {
        case Dust:
            dy+=gravity10;
            break;
        case Pixels:
            dy+=gravity2;
            break;
        case Smoke:
            life--;
        default:
            break;
        };

        //aging particle
        life--;
    }

    void draw(Camera *cam)
    {
        switch(type)
        {
        case Explosion:
            d.drawBitmap(getX()-cam->getX()-4,getY()-cam->getY()-4,gfxExplosion[animExplosion[life/5]]);
            break;
        case Bullet:
            d.setColor(12);
            d.drawLine(getX()-cam->getX(),getY()-cam->getY(),
                       fix16_to_int(Fix16(x+dx))-cam->getX(),
                       fix16_to_int(Fix16(y+dy))-cam->getY());
            break;
        case Dust:
            d.drawBitmap(getX()-cam->getX()-4,getY()-cam->getY()-4,gfxDust[life/6]);
            break;
        case Pixels:
            d.setColor(palExplosion[life/25]);
            d.drawPixel(getX()-cam->getX(),getY()-cam->getY());
            break;
        case Sparkle:
            d.setColor(palExplosion[life/2]);
            d.drawLine(getX()-cam->getX(),getY()-cam->getY(),
                       fix16_to_int(Fix16(x-dx))-cam->getX(),
                       fix16_to_int(Fix16(y-dy))-cam->getY());
            break;
        case Smoke:
            d.drawBitmap(getX()-cam->getX()-4,getY()-cam->getY()-4,gfxSmoke[life/6]);
            break;
        case Rock:
            d.drawBitmap(getX()-cam->getX()-4,getY()-cam->getY()-4,gfxRocks[5]);
            break;
        }
    }




};


#endif // PARTICLE_H_INCLUDED
