#pragma once

#include "Pokitto.h"
#include "Particle.h"
#include "Engine.h"
#include "Layer.h"

const char* stateString[]=
{
    "Idle",
    "Run",
    "Jump",
    "Fly",
    "Fall",
    "JumpDouble",
    "Fire",
    "Crunch",
    "CrunchMove"
};

class Player
{
    typedef enum
    {
        Idle,
        Run,
        Jump,
        Fall,
        JumpDouble,
        Fire,
        Crunch,
        CrunchMove,
    } State;

    typedef enum
    {
        UpLeft,
        UpRight,
        Right,
        Left,
    } Facing;

private:
    const Fix16 accRun = Fix16(0.15);
    const Fix16 speedMaxRun = Fix16(1.5);
    const Fix16 speedMinRun = Fix16(-1.5);
    const Fix16 speedCrunch = Fix16(0.3);
    const Fix16 frictionMove = Fix16(0.65);
    const Fix16 startSpeedJump = Fix16(-3.0);
    const Fix16 accJump = Fix16(0.49);
    const Fix16 speedMaxJump = Fix16(-10.0);
    const Fix16 accFall = Fix16(0.48);
    const Fix16 speedMaxFall = Fix16(4.0);
    const Fix16 accFly = Fix16(0.5);

    const Fix16 speedShoot = Fix16(4.0);

    const Fix16 angleUp = Fix16(PI);
    const Fix16 angleRight = Fix16(PI/2.0);
    const Fix16 angleUpRight = Fix16(105.0 * PI / 180.0);
    const Fix16 angleUpLeft = Fix16(255.0 * PI / 180.0);
    const Fix16 angleLeft = Fix16((PI*3.0)/2.0);

    const Fix16 angleDown = Fix16(0.0);
    const Fix16 angleDownLeft = Fix16(2.0*PI);
    const Fix16 angleDownRight = Fix16(2.0*PI);

    const Fix16 targetRadius = Fix16(20.0);

    Fix16 angleShoot = Fix16(PI/2.0);
    Fix16 x;
    Fix16 y;
    Fix16 xPrec;
    Fix16 yPrec;
    PointT<int> iPos;
    PointT<int> iCamera;
    PointT<int> iPosScreen;

    Fix16 speedX;
    Fix16 speedY;
    Fix16 speedXAbs;

    Layer::TileExp tMemDw;
    Layer::TileExp* tDwPrec = &tMemDw;

    Layer::TileExp* tUp = nullptr;
    Layer::TileExp* tDw = nullptr;
    Layer::TileExp* tSx = nullptr;
    Layer::TileExp* tDx = nullptr;
    Layer::TileExp screenTiles[12][15];//[row][columns]


    RectangleT<int16_t> pHitBox;
    RectangleT<int16_t> pHitBoxSX;
    RectangleT<int16_t> pHitBoxDX;
    RectangleT<int16_t> pHitBoxDW;
    RectangleT<int16_t> pHitBoxUP;

    State state=State::Idle;
    Fix16 msState=0.0;
    Facing facing=Facing::Right;
    Fix16 msJumpPressing=0.0;
    uint8_t subFrame;
    uint8_t frame;

public:

    Player(int x,int y)
    {
        this->x=fix16_from_int(x);
        this->y=fix16_from_int(y);
        speedX=0.0;
        speedY=0.0;
    }

    Layer::TileExp* getScreenTiles()
    {
        return *screenTiles;
    }

    int getX()
    {
        return fix16_to_int(x);
    }

    int getY()
    {
        return fix16_to_int(y);
    }

    void changeState(State newState)
    {
        msState=0.0;

        if(newState==State::Jump || newState==State::JumpDouble)
        {
            speedY = startSpeedJump;
            msJumpPressing=0.0;
        }

        state=newState;
    }

    void updatePhysic(Layer* layer)
    {
        if(tDw!=nullptr && tDwPrec!=nullptr && tDw->Actioned)
        {
            x += fix16_from_int(tDw->PositionDelta.x-tDwPrec->PositionDelta.x);
            printf("dx=%i \r\n",tDw->PositionDelta.x-tDwPrec->PositionDelta.x);
        }

        if(tDw!=nullptr)
        {
            tDwPrec=&tMemDw;
            tDwPrec->PositionDelta.x = tDw->PositionDelta.x;
            tDwPrec->PositionDelta.y = tDw->PositionDelta.y;
        }
        else
        {
            tDwPrec=nullptr;
        }

        //Collisions detection
        iPos.x=getX();
        iPos.y=getY();

        iCamera.x=layer->getCamera()->getX();
        iCamera.y=layer->getCamera()->getY();

        iPosScreen.x=iPos.x-iCamera.x;
        iPosScreen.y=iPos.y-iCamera.y;

        //Calculate tile where we are
        pHitBox.Set(iPos.x+2,iPos.y+1,iPos.x+6,iPos.y+CELL_HEIGHT);
        pHitBoxSX.Set(iPos.x+1,iPos.y+3,iPos.x+2,iPos.y+CELL_HEIGHT-3);
        pHitBoxDX.Set(iPos.x+6,iPos.y+3,iPos.x+7,iPos.y+CELL_HEIGHT-3);
        pHitBoxUP.Set(iPos.x+3,iPos.y,iPos.x+5,iPos.y);
        pHitBoxDW.Set(iPos.x+3,iPos.y+CELL_HEIGHT+1,iPos.x+5,iPos.y+CELL_HEIGHT+1);

         //Extract surrounding cells
        int xCellRem = (iCamera.x)%CELL_WIDTH;
        int yCellRem = (iCamera.y)%CELL_HEIGHT;

        int xcell=(iPosScreen.x+4+xCellRem)/CELL_WIDTH;
        int ycell=(iPosScreen.y+3+yCellRem)/CELL_HEIGHT;

        tDx=&screenTiles[ycell][xcell+1];
        tSx=&screenTiles[ycell][xcell-1];
        tUp=&screenTiles[ycell-1][xcell];
        tDw=&screenTiles[ycell+1][xcell];


        //Search sorrounding tiles
//        tDw=tUp=tDx=tSx=nullptr;
//        for(short r=0; r<12; r++)//11 rows
//        {
//            for(short c=0; c<15; c++)//14 columns
//            {
//                if(screenTiles[r][c].Alive && screenTiles[r][c].Solid)
//                {
//                    if(screenTiles[r][c].HitBox.Collide(pHitBoxDW))
//                    {
//                        tDw=&screenTiles[r][c];
//                    }
//                    if(screenTiles[r][c].HitBox.Collide(pHitBoxUP))
//                    {
//                        tUp=&screenTiles[r][c];
//                    }
//                    if(screenTiles[r][c].HitBox.Collide(pHitBoxDX))
//                    {
//                        tDx=&screenTiles[r][c];
//                    }
//                    if(screenTiles[r][c].HitBox.Collide(pHitBoxSX))
//                    {
//                        tSx=&screenTiles[r][c];
//                    }
//                }
//            }
//        }

        //Collisions
        if(tDw!=nullptr && pHitBox.Collide(tDw->HitBox) && speedY>0.0)
        {
            speedY=0.0;
            y=fix16_from_int(tDw->HitBox.y1-CELL_HEIGHT);
        }
        else
        {
            if(speedY<speedMaxFall)
                speedY+=accFall;
        }
        if(tUp!=nullptr && pHitBox.Collide(tUp->HitBox) && !tUp->OneWay && speedY<0.0)
        {
            speedY=accFall;
            y=fix16_from_int(tUp->HitBox.y2);
        }
        if(tSx!=nullptr && pHitBox.Collide(tSx->HitBox) && speedX<0.0)
        {
            x=fix16_from_int(tSx->HitBox.x2-2);
            speedX=0.0;
        }
        if(tDx!=nullptr && pHitBox.Collide(tDx->HitBox) && speedX>0.0)
        {
            x=fix16_from_int(tDx->HitBox.x1-6);
            speedX=0.0;
        }

        //Apply force
        x+=speedX;
        y+=speedY;

        //Limit force
        if(speedX>speedMaxRun)
            speedX=speedMaxRun;
        if(speedX<speedMinRun)
            speedX=speedMinRun;

        if(!g.rightBtn() && !g.leftBtn())
            speedX=speedX*frictionMove;

        speedXAbs=speedX;
        if(speedX<0.0)
            speedXAbs=Fix16(0.0)-speedXAbs;

        //Save iPos prec
        xPrec=x;
        yPrec=y;
    }

    void update(Layer* layer,Engine* engine)
    {
        subFrame++;
        if(subFrame>1)
        {
            subFrame=0;
            frame++;
        }

        //Resolve facing
        if(g.leftBtn())
        {
            facing=Facing::Left;
            angleShoot = angleLeft;
        }
        if(g.rightBtn())
        {
            facing=Facing::Right;
            angleShoot = angleRight;
        }
        if(g.upBtn())
        {
            if(facing==Facing::Right )
            {
                facing=Facing::UpRight;
                angleShoot = angleUpRight;
            }
            else if(facing==Facing::Left )
            {
                facing=Facing::UpLeft;
                angleShoot = angleUpLeft;
            }
        }
        else
        {
            if(facing==UpLeft)
                facing=Facing::Left;
            if(facing==UpRight)
                facing=Facing::Right;
        }


        switch (state)
        {
        case State::Idle:
            //RUN
            if(g.leftBtn() || g.rightBtn())
                changeState(State::Run);
            //JUMP
            if(g.buttons.pressed(BTN_A))
                changeState(State::Jump);
            //CRUNCH
            if(g.downBtn())
                changeState(State::Crunch);
            break;

        case State::Run:
            if(g.leftBtn())
            {
                speedX-=accRun;
            }
            if(g.rightBtn())
            {
                speedX+=accRun;
            }
            //JUMP
            if(g.buttons.aBtn() && speedY==0.0)
            {
                changeState(State::Jump);
            }
            if(!g.leftBtn() && !g.rightBtn())
            {
                changeState(State::Idle);
            }

            break;

        case State::Jump:
            if(g.leftBtn())
            {
                speedX-=accRun;
            }
            if(g.rightBtn())
            {
                speedX+=accRun;
            }
            //Jump rising
            if(g.aBtn() && msJumpPressing<250.0 && speedY>speedMaxJump)
            {
                speedY-=accJump;
            }
            //No more jump
            if( !g.aBtn())
                changeState(State::Fall);
            //Stopped
            if(speedY==0.0)
                changeState(State::Idle);
            break;

        case State::Fall:
            if(g.buttons.pressed(BTN_A))
            {
                speedY-=(accJump*2.0);
                engine->AddSmoke(x+fix16_from_int(CELL_HALF_WIDTH),y+fix16_from_int(CELL_HEIGHT),0);
                changeState(State::JumpDouble);
            }
            //Stopped
            if(speedY==0.0)
                changeState(State::Idle);
            break;

        case State::JumpDouble:

            //Jump rising
            if(g.aBtn() && msJumpPressing<350.0 && speedY>speedMaxJump)
            {
                speedY-=accJump;
                if(frame%3==0)
                    engine->AddSmoke(x+fix16_from_int(CELL_HALF_WIDTH),y+fix16_from_int(CELL_HEIGHT),0);
            }
            else
                changeState(State::Idle);
            //Stopped
            if(speedY==0.0)
                changeState(State::Idle);
            break;

        case State::Crunch:
            if(g.leftBtn() || g.rightBtn())
                changeState(State::CrunchMove);
            if(g.upBtn())
                changeState(State::Idle);
            break;

        case State::CrunchMove:
            if(g.leftBtn())
            {
                speedX-=speedCrunch;
            }
            if(g.rightBtn())
            {
                speedX+=speedCrunch;
            }
            if(!g.leftBtn() && !g.rightBtn())
                changeState(State::Crunch);
            if(g.upBtn())
                changeState(State::Idle);
            break;

        default:
            break;
        }


        //SHOT
        if(g.buttons.pressed(BTN_B))
        {
            Particle p=Particle(x+fix16_from_int(4),
                                y+fix16_from_int(4),
                                speedShoot+speedXAbs,
                                angleShoot,
                                60,
                                Particle::Bullet);
            engine->AddParticle(p);
        }

        //
        updatePhysic(layer);

        //Increase times
        msState+=MS_FRAME;
        msJumpPressing+=MS_FRAME;
    }

    void draw(Layer* layer)
    {

        //Draw sprite
        switch (state)
        {
        case State::Idle:
            if(facing==Left || facing==UpLeft)
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[6],0,1);
            else
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[6]);
            break;
        case State::Run:
            if(facing==Left || facing==UpLeft)
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animRun[frame%7]],0,1);
            else
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animRun[frame%7]]);
            break;
        case State::Jump:
        case State::Fall:
        case State::JumpDouble:
            if(facing==Left)
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[3],0,1);
            else
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[3]);
            break;
        case State::Crunch:
            if(facing==Left)
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animCrunch[0]],0,1);
            else
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animCrunch[0]]);
            break;
        case State::CrunchMove:
            if(facing==Left)
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animCrunch[frame/2%4]],0,1);
            else
                d.drawBitmap(iPosScreen.x,iPosScreen.y,gfxPlayer[animCrunch[frame/2%4]]);
            break;

        default:
            break;
        }



        // for(short r=0; r<12; r++)//11 rows
        // {
        //     for(short c=0; c<15; c++)//14 columns
        //     {
        //         d.setColor(12);
        //         d.drawRect(screenTiles[r][c].HitBox.x1-iCamera.x,screenTiles[r][c].HitBox.y1-iCamera.y,8,8);
        //     }
        // }




        // d.color = 9;
        // if(tDx!=nullptr)
        //     d.drawRect(tDx->HitBox.x1-iCamera.x,
        //               tDx->HitBox.y1-iCamera.y,
        //               4,
        //               4);

        // d.color = 6;
        // if(tSx!=nullptr)
        //     d.drawRect(tSx->HitBox.x1-iCamera.x,
        //               tSx->HitBox.y1-iCamera.y,
        //               4,
        //               4);

        // d.color = 12;
        // if(tDw!=nullptr)
        //     d.drawRect(tDw->HitBox.x1-iCamera.x,
        //               tDw->HitBox.y1-iCamera.y,
        //               4,
        //               4);

        // d.color = 13;
        // if(tUp!=nullptr)
        //     d.drawRect(tUp->HitBox.x1-iCamera.x,
        //               tUp->HitBox.y1-iCamera.y,
        //               4,
        //               4);



//        d.setColor(0);
//        d.drawRect(pHitBox.x1-iCamera.x,
//                   pHitBox.y1-iCamera.y,
//                   (pHitBox.x2-pHitBox.x1),
//                   (pHitBox.y2-pHitBox.y1));
//        d.drawRect(pHitBoxDW.x1-iCamera.x,
//                   pHitBoxDW.y1-iCamera.y,
//                   (pHitBoxDW.x2-pHitBoxDW.x1),
//                   (pHitBoxDW.y2-pHitBoxDW.y1));
//        d.drawRect(pHitBoxUP.x1-iCamera.x,
//                   pHitBoxUP.y1-iCamera.y,
//                   (pHitBoxUP.x2-pHitBoxUP.x1),
//                   (pHitBoxUP.y2-pHitBoxUP.y1));
//        d.drawRect(pHitBoxSX.x1-iCamera.x,
//                   pHitBoxSX.y1-iCamera.y,
//                   (pHitBoxSX.x2-pHitBoxSX.x1),
//                   (pHitBoxSX.y2-pHitBoxSX.y1));
//        d.drawRect(pHitBoxDX.x1-iCamera.x,
//                   pHitBoxDX.y1-iCamera.y,
//                   (pHitBoxDX.x2-pHitBoxDX.x1),
//                   (pHitBoxDX.y2-pHitBoxDX.y1));





//        //Debug
        d.setColor(141);
        d.setCursor(0,8);
        d.print(stateString[state]);




    }

};

