#include "Pokitto.h"
#include "math.h"

#include "..\FixMath\fixmath.h"

#define WIDTH_HALF 55
#define HEIGHT_HALF 44
#define FPS 40
#define MS_FRAME 1000.0/FPS
#define CELL_WIDTH 8
#define CELL_HEIGHT 8
#define CELL_HALF_WIDTH CELL_WIDTH/2
#define CELL_HALF_HEIGHT CELL_HEIGHT/2


Pokitto::Core g;
Pokitto::Display d;

const int8_t animExplosion[] = {14,13,12,11,10,9,8,7,6,5};
const int8_t animRun[] = {1,2,3,4,5,6,7};
const int8_t animCrunch[] = {10,11,12,13};
const int8_t palExplosion[] = {5,6,11,12,1};

const Fix16 gravity=Fix16(0.9);
const Fix16 gravity2=Fix16(0.45);
const Fix16 gravity10=Fix16(0.09);
const Fix16 toRad=Fix16(PI/180.0);
const Fix16 toDegrees=Fix16(180.0/PI);

#include "gfx.h"
#include "gfxPlayer.h"
#include "gfxSmoke.h"
#include "gfxExplosion.h"
#include "gfxDust.h"
#include "gfxRocks.h"
#include "gfxTest.h"

#include "PointT.h"
#include "RectangleT.h"
#include "Camera.h"
#include "Player.h"
#include "Layer.h"
#include "Engine.h"


Layer* layer0;
Layer* layer1;
Layer* layer2;

Player* player;
Engine* engine;


void init()
{
    player=new Player(80,40);
    engine=new Engine();

    layer0= new Layer(new Camera(0,0,3),&layer_config_L0,&layer_New_L0[0][0],
                      &layer_animations_L0[0],nullptr,nullptr,nullptr);
    layer1= new Layer(new Camera(0,0,2),&layer_config_L1,&layer_New_L1[0][0],
                      &layer_animations_L1[0],nullptr,nullptr,nullptr);
    layer2= new Layer(new Camera(0,0,1),&layer_config_L2,&layer_New_L2[0][0],
                      &layer_animations_L2[0],&layer_damage_L2[0],&layer_actions_L2[0],player->getScreenTiles());

};

int main ()
{
    g.begin();
    d.setFont(fontTight);
    d.load565Palette(pal);
    d.setInvisibleColor(0);
    d.bgcolor=15;
    g.setFrameRate(FPS);

    init();

    while (g.isRunning())
    {
        if (g.update())
        {
            int xP=player->getX() - WIDTH_HALF;
            int yP=player->getY() - HEIGHT_HALF;

            //Cam point to players at center screen
            layer0->setCameraTarget(xP,yP);
            layer0->update();
            layer0->draw();

            layer1->setCameraTarget(xP,yP);
            layer1->update();
            layer1->draw();

            layer2->setCameraTarget(xP,yP);
            layer2->update();
            layer2->draw();

            engine->update(layer2);
            engine->draw(layer2);

            player->update(layer2,engine);
            player->draw(layer2);

            //Light effect (Pixel per Pixel palette shift...)
            int x,y,ab,o=0;
            int l=4;
            if(g.cBtn())
                l=6;
            for ( y=0; y<88; y++)
            {
                ab=abs(44-y)/l*16;
                for ( x=0; x<110; x++)
                {
                    o=(y*110)+(x);
                    d.screenbuffer[o]+=(240-ab);
                }
            }

            //FPS
            d.setColor(141);
            d.setCursor(0,0);
            d.print(g.fps_counter);

        }
    }
    return 0;
}

